@@ -1,5 +1,3 @@
-import warnings
-
 import torch
 import torch.nn as nn
 from mmcv.cnn import ConvModule, kaiming_init
@@ -11,10 +9,10 @@ from ..registry import BACKBONES
 from .resnet3d import ResNet3d
 
 try:
-    import mmdet  # noqa
     from mmdet.models import BACKBONES as MMDET_BACKBONES
+    mmdet_imported = True
 except (ImportError, ModuleNotFoundError):
-    warnings.warn('Please install mmdet to use MMDET_BACKBONES')
+    mmdet_imported = False
 
 
 class ResNet3dPathway(ResNet3d):
@@ -509,5 +507,5 @@ class ResNet3dSlowFast(nn.Module):
         return out
 
 
-if 'mmdet' in dir():
+if mmdet_imported:
     MMDET_BACKBONES.register_module()(ResNet3dSlowFast)
