@@ -1,3 +1,5 @@
+import torch
+
 from ..registry import RECOGNIZERS
 from .base import BaseRecognizer
 
@@ -29,13 +31,29 @@ class Recognizer3D(BaseRecognizer):
         num_segs = imgs.shape[1]
         imgs = imgs.reshape((-1, ) + imgs.shape[2:])
 
-        x = self.extract_feat(imgs)
-        if hasattr(self, 'neck'):
-            x, _ = self.neck(x)
+        if self.max_testing_views is not None:
+            total_views = imgs.shape[0]
+            assert num_segs == total_views, (
+                'max_testing_views is only compatible '
+                'with batch_size == 1')
+            view_ptr = 0
+            cls_scores = []
+            while view_ptr < total_views:
+                batch_imgs = imgs[view_ptr:view_ptr + self.max_testing_views]
+                x = self.extract_feat(batch_imgs)
+                if hasattr(self, 'neck'):
+                    x, _ = self.neck(x)
+                cls_score = self.cls_head(x)
+                cls_scores.append(cls_score)
+                view_ptr += self.max_testing_views
+            cls_score = torch.cat(cls_scores)
+        else:
+            x = self.extract_feat(imgs)
+            if hasattr(self, 'neck'):
+                x, _ = self.neck(x)
+            cls_score = self.cls_head(x)
 
-        cls_score = self.cls_head(x)
         cls_score = self.average_clip(cls_score, num_segs)
-
         return cls_score
 
     def forward_test(self, imgs):
