@@ -1,4 +1,5 @@
 from .collect_env import collect_env
+from .decorators import import_module_error_class, import_module_error_func
 from .gradcam_utils import GradCAM
 from .logger import get_root_logger
 from .misc import get_random_string, get_shm_dir, get_thread_id
@@ -6,5 +7,6 @@ from .precise_bn import PreciseBNHook
 
 __all__ = [
     'get_root_logger', 'collect_env', 'get_random_string', 'get_thread_id',
-    'get_shm_dir', 'GradCAM', 'PreciseBNHook'
+    'get_shm_dir', 'GradCAM', 'PreciseBNHook', 'import_module_error_class',
+    'import_module_error_func'
 ]
